package com.ningdatech.pmapi.meeting.helper;

import cn.hutool.core.lang.Assert;
import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.ningdatech.basic.exception.BizException;
import com.ningdatech.basic.util.CollUtils;
import com.ningdatech.pmapi.common.util.BizUtils;
import com.ningdatech.pmapi.expert.constant.ExpertAccountStatusEnum;
import com.ningdatech.pmapi.expert.entity.ExpertUserFullInfo;
import com.ningdatech.pmapi.expert.service.ExpertInfoService;
import com.ningdatech.pmapi.expert.service.IExpertUserFullInfoService;
import com.ningdatech.pmapi.meeting.entity.domain.ExpertInviteAvoidRule;
import com.ningdatech.pmapi.meeting.entity.domain.Meeting;
import com.ningdatech.pmapi.meeting.entity.domain.MeetingExpert;
import com.ningdatech.pmapi.meeting.entity.dto.AvoidInfoDTO;
import com.ningdatech.pmapi.meeting.entity.dto.MeetingAndAttendStatusDTO;
import com.ningdatech.pmapi.meeting.entity.dto.MeetingBasicDTO;
import com.ningdatech.pmapi.meeting.entity.enumeration.ExpertAttendStatus;
import com.ningdatech.pmapi.meeting.entity.enumeration.MeetingStatus;
import com.ningdatech.pmapi.meeting.entity.req.MeetingListReq;
import com.ningdatech.pmapi.meeting.entity.vo.ExpertBasicInfoVO;
import com.ningdatech.pmapi.meeting.entity.vo.MeetingByManagerVO;
import com.ningdatech.pmapi.meeting.service.IExpertInviteAvoidRuleService;
import com.ningdatech.pmapi.meeting.service.IExpertInviteRuleService;
import com.ningdatech.pmapi.meeting.service.IMeetingExpertService;
import com.ningdatech.pmapi.meta.helper.DictionaryCache;
import lombok.AllArgsConstructor;
import org.apache.commons.collections4.CollectionUtils;
import org.springframework.stereotype.Component;

import java.util.*;

/**
 * <p>
 * MeetingManageHelper
 * </p>
 *
 * @author WendyYang
 * @since 17:23 2022/8/23
 */
@Component
@AllArgsConstructor
public class MeetingManageHelper {

    private final DictionaryCache dictionaryCache;
    private final ExpertInfoService expertInfoService;
    private final IExpertUserFullInfoService expertUserFullInfoService;
    private final IMeetingExpertService meetingExpertService;
    private final IExpertInviteRuleService inviteRuleService;
    private final IExpertInviteAvoidRuleService inviteAvoidRuleService;

    public void checkReviewProject(MeetingBasicDTO meetingBasic) {
        if (meetingBasic.getIsInnerProject()) {
            Assert.notEmpty(meetingBasic.getProjectIdList(), "评审项目不能为空");
        } else {
            Assert.notEmpty(meetingBasic.getProjects(), "评审项目不能为空");
        }
    }

    /**
     * 获取专家出席会议的状态
     *
     * @param info 会议状态及评价信息
     * @return java.lang.Integer
     * @author WendyYang
     **/
    public Integer getExpertAttendStatus(MeetingAndAttendStatusDTO info) {
        if (info.getAttended() == null && info.getStatus().equals(ExpertAttendStatus.AGREED.getCode())) {
            return MeetingStatus.Expert.TO_ATTEND.getCode();
        } else if (info.getStatus().equals(ExpertAttendStatus.ON_LEAVE.getCode())) {
            return MeetingStatus.Expert.ON_LEAVE.getCode();
        } else if (info.getAttended() != null && info.getAttended()) {
            return MeetingStatus.Expert.ATTENDED.getCode();
        } else {
            return MeetingStatus.Expert.UN_ATTEND.getCode();
        }
    }

    public MeetingByManagerVO buildByMeeting(Meeting meeting) {
        return MeetingByManagerVO.builder()
                .id(meeting.getId())
                .dateTypeName(dictionaryCache.getByCode(meeting.getType()).getName())
                .meetingAddress(meeting.getMeetingAddress())
                .name(meeting.getName())
                .startTime(meeting.getStartTime())
                .endTime(meeting.getEndTime())
                .connecter(meeting.getConnecter())
                .contact(meeting.getContact())
                .status(meeting.getStatus())
                .holdOrg(meeting.getHoldOrg())
                .build();
    }

    public void buildMeetingQuery(LambdaQueryWrapper<Meeting> query, MeetingListReq po) {
        if (StrUtil.isNotBlank(po.getName())) {
            query.like(Meeting::getName, po.getName());
        }
        if (StrUtil.isNotBlank(po.getType())) {
            query.eq(Meeting::getType, po.getType());
        }
        if (po.getStartTime() != null) {
            query.ge(Meeting::getStartTime, po.getStartTime());
        }
        if (po.getEndTime() != null) {
            query.le(Meeting::getEndTime, po.getEndTime());
        }
    }

    /**
     * 构建用户基本信息
     *
     * @param userIds 用户ID
     * @return java.util.Map<java.lang.Long, com.ningdatech.pmapi.meeting.entity.vo.ExpertBasicInfoVo>
     * @author WendyYang
     **/
    public Map<Long, ExpertBasicInfoVO> getExpertBasicInfo(List<Long> userIds) {
        /*List<ExpertFullInfoAllDTO> expertInfos = expertInfoService.listExpertUserFullInfoAll(userIds);
        return CollUtils.listToMap(expertInfos, ExpertFullInfoAllDTO::getUserId, w -> {
            ExpertBasicInfoVO basicInfoVo = new ExpertBasicInfoVO();
            ExpertUserFullInfoDTO userInfo = w.getExpertUserInfoDTO();
            basicInfoVo.setName(userInfo.getName());
            basicInfoVo.setExpertId(w.getUserId());
            basicInfoVo.setContact(userInfo.getPhoneNo());
            basicInfoVo.setCompany(userInfo.getCompany());
            Optional<ExpertDictionaryDTO> first = w.getExpertDictionaryList().stream()
                    .filter(dict -> dict.getExpertInfoField().equals(DictExpertInfoTypeEnum.TITLE_LEVEL.getKey()))
                    .findFirst();
            if (first.isPresent()) {
                DictionaryDTO dictInfo = dictionaryCache.getByCode(first.get().getDictionaryCode());
                basicInfoVo.setJobLevel(dictInfo.getName());
            } else {
                basicInfoVo.setJobLevel("");
            }
            basicInfoVo.setCompanyType("");
            return basicInfoVo;
        });*/
        // TODO
        return null;
    }

    public AvoidInfoDTO getAvoidInfoDto(Long meetingId) {
        ExpertInviteAvoidRule avoidRule = inviteAvoidRuleService.getByMeetingId(meetingId);
        AvoidInfoDTO result = new AvoidInfoDTO();
        result.setAvoidOrgIdList(StrUtil.split(avoidRule.getAvoidOrgIds(), ","));
        result.setExpertIds(BizUtils.splitToLong(avoidRule.getExpertIds()));
        result.setAvoidUnitIdList(StrUtil.split(avoidRule.getAvoidUnitIds(), ","));
        result.setWeekInviteCount(result.getWeekInviteCount());
        return result;
    }

    public void saveAvoidInfo(Long meetingId, List<Long> expertIds) {
        ExpertInviteAvoidRule avoidRule = inviteAvoidRuleService.getByMeetingId(meetingId);
        avoidRule.setExpertIds(CollUtils.joinByComma(expertIds));
        inviteAvoidRuleService.saveOrUpdate(avoidRule);
    }

    /**
     * 校验是否能够进行指定邀请
     *
     * @param meetingId 会议ID
     * @param expertIds 专家ID
     * @author WendyYang
     **/
    public List<ExpertUserFullInfo> appointExpertCheck(Long meetingId, List<Long> expertIds) {
        List<ExpertUserFullInfo> experts = expertUserFullInfoService.listByUserId(expertIds);
        AvoidInfoDTO avoidRule = getAvoidInfoDto(meetingId);
        Map<String, Integer> countMap = new HashMap<>(16);
        experts.forEach(expert -> {
            if (avoidRule.getAvoidUnitIdList().contains(expert.getCompany())) {
                throw BizException.wrap("回避单位的专家不可出现在指定邀请名单中");
            }
            if (CollectionUtils.isNotEmpty(avoidRule.getExpertIds()) && avoidRule.getExpertIds().contains(expert.getUserId())) {
                throw BizException.wrap("已回避的专家不可被指定邀请");
            }
            // 校验专家状态
            ExpertAccountStatusEnum accountStatus = ExpertAccountStatusEnum.of(expert.getExpertAccountStatus());
            if (!accountStatus.equals(ExpertAccountStatusEnum.AVAILABLE)) {
                throw BizException.wrap("专家%s不可被抽取", expert.getExpertName());
            }
        });
        Map<Long, ExpertUserFullInfo> expertMap = CollUtils.listToMap(experts, ExpertUserFullInfo::getUserId);
        List<MeetingExpert> meetingExperts = meetingExpertService.listByMeetingId(meetingId);
        if (!meetingExperts.isEmpty()) {
            Comparator<MeetingExpert> comparator = Comparator.comparing(MeetingExpert::getUpdateOn).reversed();
            Collection<MeetingExpert> values = BizUtils.groupFirst(meetingExperts, MeetingExpert::getExpertId, comparator);
            values.forEach(w -> {
                ExpertUserFullInfo expertInfo = expertMap.get(w.getExpertId());
                if (expertInfo != null) {
                    String expertName = expertInfo.getExpertName();
                    switch (ExpertAttendStatus.getByCode(w.getStatus())) {
                        case REFUSED:
                            throw BizException.wrap("专家%s已拒绝参加", expertName);
                        case CANCELED:
                            throw BizException.wrap("专家%s已被移除", expertName);
                        case REPLACED:
                            switch (ExpertAttendStatus.getByCode(w.getPreStatus())) {
                                case REFUSED:
                                    throw BizException.wrap("专家%s已拒绝参加", expertName);
                                case CANCELED:
                                    throw BizException.wrap("专家%s已被移除", expertName);
                                default:
                                    break;
                            }
                            break;
                        case AGREED:
                            throw BizException.wrap("专家%s已同意参加", expertName);
                        case NOTICING:
                            throw BizException.wrap("专家%s正在通知中", expertName);
                        default:
                            break;
                    }
                }
            });
        }
        return experts;
    }

}
